#
# Copyright 2022 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from __future__ import print_function

import os, subprocess, sys
from functools import partial

from .qt import QtWidgets

class ConfigController:
    def __init__(self, currentConfig, appController):
        self._appController = appController
        self._currentConfig = currentConfig

        # XXX Temporarily hiding behind env variable until ready for public
        if os.getenv('USDVIEWQ_CONFIG_CONTROLLER') is None:
            self._hide(self._appController._ui)
        else:
            self.reloadConfigController()

    def _hide(self, ui):
        ui.menuLoad_New_State.menuAction().setVisible(False)
        ui.actionSave_State_To.setVisible(False)
        ui.menuSave_State_As.menuAction().setVisible(False)
        ui.actionSave_State_As_New_Config.setVisible(False)

    def reloadConfigController(self):
        """Can be used for refreshing UI if current config changes"""
        ui = self._appController._ui
        configs = self._appController._configManager.getConfigs()[1:]

        if configs:
            for config in configs:
                ui.menuLoad_New_State.addAction(config).triggered.connect(
                    partial(subprocess.Popen, [
                        sys.argv[0],
                        self._appController._parserData.usdFile,
                        '--config', config]))
            ui.menuLoad_New_State.menuAction().setVisible(True)
        else:
            ui.menuLoad_New_State.menuAction().setVisible(False)

        save = ui.actionSave_State_To
        if self._currentConfig:
            save.setText(save.text() + self._currentConfig)
            save.triggered.connect(self._appController._configManager.save)
            save.setVisible(True)
        else:
            save.setVisible(False)

        if configs:
            for config in configs:
                ui.menuSave_State_As.addAction(config).triggered.connect(
                    partial(self._appController._configManager.save, config))
            ui.menuSave_State_As.menuAction().setVisible(True)
        else:
            ui.menuSave_State_As.menuAction().setVisible(False)

        ui.actionSave_State_As_New_Config.triggered.connect(
            lambda: self._saveAsTriggered())

        # XXX When no longer hiding controller behind env variable, move this
        # separator to the static ui file
        ui.menuFile.insertSeparator(ui.actionQuit)

    def _validateAndSaveConfig(self, newName, dialog):
        if not newName:
            print("Invalid config name, not saving", file=sys.stderr)
            return
        self._appController._configManager.save(newName)
        dialog.close()

    def _saveAsTriggered(self):
        configDialog = QtWidgets.QDialog(self._appController._mainWindow)
        configDialog.setWindowTitle("Save State As")

        layout = QtWidgets.QHBoxLayout()
        field = QtWidgets.QLineEdit(self._currentConfig)
        field.textEdited.connect(lambda text: field.setText(text.lower()))
        layout.addWidget(field)

        fieldsLayout = QtWidgets.QVBoxLayout()
        fieldsLayout.addWidget(QtWidgets.QLabel(
            "Config Name"))
        fieldsLayout.addLayout(layout)
        fieldsLayout.addStretch()

        buttonBox = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.StandardButton.Cancel | QtWidgets.QDialogButtonBox.StandardButton.Save)
        buttonBox.rejected.connect(configDialog.close)
        buttonBox.accepted.connect(lambda :
            self._validateAndSaveConfig(field.text(), configDialog))

        configDialogLayout = QtWidgets.QVBoxLayout()
        configDialogLayout.addLayout(fieldsLayout)
        configDialogLayout.addWidget(buttonBox)
        configDialog.setLayout(configDialogLayout)

        configDialog.open()


#
# Copyright 2019 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

def AddCmdlineArgs(argsParser, defaultValue='sRGB', altHelpText=''):
    """
    Adds color-related command line arguments to argsParser.

    The resulting 'colorCorrectionMode' argument will be a Python string.
    """
    helpText = altHelpText
    if not helpText:
        helpText = (
            'the color correction mode to use (default=%(default)s)')

    argsParser.add_argument('--colorCorrectionMode', '-color', action='store',
        type=str, default=defaultValue,
        choices=['disabled', 'sRGB', 'openColorIO'], help=helpText)



#
# Copyright 2018 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

# Execute a script with tracing enabled.

# Usage: python -m pxr.Trace [-o OUTPUT] [-c "command" | FILE]

from __future__ import print_function
import argparse, sys

parser = argparse.ArgumentParser(description='Trace script execution.')
parser.add_argument('-o', dest='output', metavar='OUTPUT',
                    type=str, default=None,
                    help='trace output; defaults to stdout')
parser.add_argument('-c', dest='cmd',
                    type=str,
                    help='trace <cmd> as a Python script')
parser.add_argument('file', metavar="FILE",
                    nargs='?',
                    default=None,
                    help='script to trace')

args = parser.parse_args()

if not args.cmd and not args.file:
    print("Must specify a command or script to trace", file=sys.stderr)
    sys.exit(1)

if args.cmd and args.file:
    print("Only one of -c or FILE may be specified", file=sys.stderr)
    sys.exit(1)

from pxr import Trace

env = {}

# Create a Main function that is traced so we always capture a useful
# top-level scope.
if args.file:
    @Trace.TraceFunction
    def Main():
        exec(compile(open(args.file).read(), args.file, 'exec'), env)
else:
    @Trace.TraceFunction
    def Main():
        exec(args.cmd, env)

try:
    Trace.Collector().enabled = True
    Main()
finally:
    Trace.Collector().enabled = False

    if args.output is not None:
        Trace.Reporter.globalReporter.Report(args.output)
    else:
        Trace.Reporter.globalReporter.Report()



#
# Copyright 2022 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
import os
import platform
import sys
import shutil

def FindUsdBinary(name):
    """Returns the full path to the named executable if it can be found, or
    None if the executable cannot be located. This first searches in PATH, and
    if the executable is not found, it then searches in the parent directory
    of the current process, as identified by sys.argv[0].

    On Windows, this function searches for both name.EXE and name.CMD to
    ensure that CMD-wrapped executables are located if they exist.
    """

    # First search PATH
    binpath = shutil.which(name)
    if binpath:
        return binpath

    # Then look relative to the current executable
    binpath = shutil.which(name,
        path=os.path.abspath(os.path.dirname(sys.argv[0])))
    if binpath:
        return binpath

    if platform.system() == 'Windows':
        # shutil.which under Windows only returns *.EXE files so we need to
        # traverse the tool path.
        path = os.environ.get('PATH', '').split(os.pathsep)
        for base in [os.path.join(p, name) for p in path]:
            # We need to test for name.cmd first because on Windows, the USD
            # executables are wrapped due to lack of UNIX style shebang support.
            for binpath in [base + ext for ext in ('.cmd', '')]:
                if os.access(binpath, os.X_OK):
                    return binpath

    return None



#
# Copyright 2021 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from pxr import Tf

def _NoArgs():
    return "_NoArgs result"

def _ReturnInt():
    return 42

def _RepeatStrings(stringList, numRepeats):
    return [s * numRepeats for s in stringList]

def _ConcatWithList(arg1, arg2, *args):
    return str.join(
        " ",
        [str(a) for a in
            [arg1, arg2]
            + list(args)])

def _ConcatWithKwArgs(arg1, arg2, arg3 = "c", arg4 = "d", **kwargs):
    return str.join(
        " ",
        [str(a) for a in
            [arg1, arg2, arg3, arg4]
            + ["%s=%s" % (k, kwargs[k]) for k in sorted(kwargs.keys())]])

def _GetTheeToANonery(arg):
    assert arg is None

_globalVar = 88

def _RaiseException():
    raise Exception("test exception")

def _RaiseTfError():
    Tf.RaiseRuntimeError("test error")



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Tf
Tf.PreparePythonModule()
del Tf

from .updateSchemaWithSdrNode import UpdateSchemaWithSdrNode, \
        SchemaDefiningKeys, SchemaDefiningMiscConstants, PropertyDefiningKeys
from .fixBrokenPixarSchemas import FixBrokenPixarSchemas
from .usdzUtils import (ExtractUsdzPackage, UsdzAssetIterator,
                        UsdzScanIterator, UsdzUpdateIterator)
